
#include "ADS1115.h"

#define SET_ACQUISITION 100
#define STOP_ACQUISITION 101
//#define DEBUG
#define N 200 // taille maximale du tampon
#define NF 21 // taille maximale du filtre convolutif
ADS1115 adc0(ADS1115_DEFAULT_ADDRESS);
const int alertReadyPin = 2;
    		   
uint32_t taille_tampon;
uint16_t x0,x1;
uint16_t tampon[2][N];
int16_t i_tampon;
uint8_t n_tampon;
bool tampon_plein;
bool acquisition;
float fact_conv;
uint8_t mux;
uint8_t gain;
uint8_t rate;
bool conversion;
float b[NF];
uint16_t tampon_filtre[NF];
int16_t i_tamp_filtre;
int16_t n_filtre = 21;
bool filtrage;
int8_t reduc;
int8_t i_reduc;
    		   
void data_ready() {
    if (acquisition) {
      if (filtrage) {
        int16_t j,k;
        float accum;
        tampon_filtre[i_tamp_filtre] = x0;
        j = i_tamp_filtre;
        accum = 0.0;
        for (k=0; k<n_filtre; k++) {
            accum += b[k]*tampon_filtre[j];
            j -= 1;
            if (j<0) j = n_filtre-1;
        }
        i_tamp_filtre += 1;
        if (i_tamp_filtre==n_filtre) i_tamp_filtre = 0;
        x0 = accum;
      }
    i_reduc += 1;
    if (i_reduc == reduc) {
        i_reduc = 0;
        tampon[n_tampon][i_tampon] = x0;
        i_tampon += 1;
        if (i_tampon==taille_tampon) {
            i_tampon = 0;
            tampon_plein = true;
            n_tampon += 1;
            if (n_tampon==2) n_tampon = 0;
          }
        }
    }
    conversion = true;
}   		       
    		       
void calcul_filtre(int nf, float fc) {
  int P = (nf-1)/2;
  int k;
  float u;
  int i;
  for (i=0; i<nf; i++) {
      k = i-P;
      u = 2*PI*k*fc;
      if (u==0) b[i] = 2*fc;
      else b[i] = 2*fc*sin(2*PI*k*fc)/(2*PI*k*fc);
      b[i] *= 0.54+0.46*cos(2*PI*k/(2*P)); // Hamming
      //b[i] *= 0.5+0.5*cos(2*PI*k/(2*P)); // Hann
  }
}    		     
    		     
void lecture_acquisition() {
   uint32_t c1,c2;
   while (Serial.available()<5) {};
    mux = Serial.read();
    gain = Serial.read();
    rate = Serial.read();
    c1 = Serial.read();
    c2 = Serial.read();
    taille_tampon = (c1<<8) | c2;
    if (taille_tampon > N) taille_tampon = N;
   while (Serial.available()<3) {};
   n_filtre = Serial.read();
   if (n_filtre>NF) n_filtre = NF;
   c1 = Serial.read();
   float fc = 1.0/c1;
   reduc = Serial.read();
   i_reduc = 0;
   if (n_filtre!=0) {
      calcul_filtre(n_filtre,fc);
      filtrage = true;
   }
   else filtrage = false;
   adc0.setMode(ADS1115_MODE_CONTINUOUS);
   adc0.setConversionReadyPinMode();
   adc0.setMultiplexer(mux);
   adc0.setRate(rate);
   adc0.setGain(gain);
   i_tampon = 0;
   n_tampon = 0;
   tampon_plein = false;
   adc0.triggerConversion();
   acquisition = true;
   conversion = true;
}
void stop_acquisition() {
  acquisition = false;
}
    		      
void setup() {
  Wire.begin();
    char c;
    Serial.begin(115200); 
    Serial.setTimeout(0);
    c = 0;
    Serial.write(c);
    c = 255;
    Serial.write(c);
    c = 0;
    Serial.write(c);
    acquisition = false;
    pinMode(alertReadyPin,INPUT_PULLUP);
    attachInterrupt(digitalPinToInterrupt(alertReadyPin),data_ready,FALLING);
    adc0.initialize(); 
    
#ifdef DEBUG
	i_tampon = 0;
    taille_tampon = 16;
    tampon_plein = false;
    i_tamp_filtre = 0;
    calcul_filtre(21,0.1);
    filtrage = true;
    reduc = 1;
    i_reduc = 0;
    adc0.setConversionReadyPinMode();
    adc0.setMode(ADS1115_MODE_CONTINUOUS);
    adc0.setRate(ADS1115_RATE_16); // 16 Hz
    adc0.setGain(ADS1115_PGA_6P144); // pas d'amplification (tension entre 0 et VDD)
    fact_conv = ADS1115_MV_6P144;
    adc0.setMultiplexer(ADS1115_MUX_P0_NG);
    acquisition = true;
    adc0.triggerConversion();
    conversion = true;
#endif
}	       
    		       
void loop() {
   char com;
   uint8_t nt;
   if (Serial.available()>0) {
      com = Serial.read();
      if (com==SET_ACQUISITION) lecture_acquisition();
       else if (com==STOP_ACQUISITION) stop_acquisition();
   }
   
   if (acquisition) {
     if (conversion) {
        x0= adc0.getConversion(false);
        conversion = false;
     }
     if (tampon_plein) {
        if (n_tampon==0) nt=1; else nt=0;
        tampon_plein = false;
#ifdef DEBUG
        for (int i=0; i<taille_tampon; i++) {
            Serial.println(tampon[nt][i]*fact_conv);
        }     
#else      
        Serial.write((uint8_t *)tampon[nt],2*taille_tampon);
         
#endif
     }
   }
}   		       
    		       